@@ -3,7 +3,7 @@ package threads::lite;
 use strict;
 use warnings;
 
-our $VERSION = '0.021';
+our $VERSION = '0.022';
 
 use 5.010;
 
@@ -14,8 +14,7 @@ use Data::Dumper;
 use XSLoader;
 XSLoader::load('threads::lite', $VERSION);
 
-##no critic qw(ProhibitAutomaticExportation)
-our @EXPORT = qw/receive receive_nb receive_table self/;
+our @EXPORT_OK = qw/spawn receive receive_nb receive_table receive_table_nb self/;
 
 require threads::lite::tid;
 
@@ -29,7 +28,6 @@ sub _deep_equals {
 	my ($message, $criterion) = @_;
 	return if $#{$message} < $#{$criterion};
 	for my $i (0..$#{$criterion}) {
-		no warnings 'uninitialized';
 		return if not $message->[$i] ~~ $criterion->[$i];
 	}
 	return 1;
@@ -49,11 +47,6 @@ sub _match_mailbox {
 	return;
 }
 
-sub spawn {
-	my ($class, $options, $sub) = @_;
-	return $class->_create($options, $sub);
-}
-
 ##no critic (Subroutines::RequireFinalReturn)
 
 sub receive {
@@ -61,7 +54,8 @@ sub receive {
 	if (my @ret = _match_mailbox(\@args)) {
 		return @ret;
 	}
-	while (my @next = _receive) {
+	while (1) {
+		my @next = _receive;
 		return _return_elements(@next) if _deep_equals(\@next, \@args);
 		push @message_cache, \@next;
 	}
@@ -72,9 +66,10 @@ sub receive_nb {
 	if (my @ret = _match_mailbox(\@args)) {
 		return @ret;
 	}
-	return if not my @next = _receive_nb;
-	return _return_elements(@next) if _deep_equals(\@next, \@args);
-	push @message_cache, \@next;
+	while (my @next = _receive_nb) {
+		return _return_elements(@next) if _deep_equals(\@next, \@args);
+		push @message_cache, \@next;
+	}
 	return;
 }
 
@@ -84,6 +79,30 @@ sub receive_table {
 
 	push @table, [ splice @args, 0, 2 ] while @args >= 2;
 
+	for my $pair (@table) {
+		if (my @ret = _match_mailbox($pair->[0])) {
+			$pair->[1]->(@ret) if defined $pair->[1];
+			return _return_elements(@ret);
+		}
+	}
+	while (1) {
+		my @next = _receive;
+		for my $pair (@table) {
+			if (_deep_equals(\@next, $pair->[0])) {
+				$pair->[1]->(@next) if defined $pair->[1];
+				return _return_elements(@next);
+			}
+		}
+		push @message_cache, \@next;
+	}
+}
+
+sub receive_table_nb {
+	my @args = @_;
+	my @table;
+
+	push @table, [ splice @args, 0, 2 ] while @args >= 2;
+
 	for my $pair (@table) {
 		if (my @ret = _match_mailbox($pair->[0])) {
 			$pair->[1]->(@ret) if defined $pair->[1];
@@ -99,6 +118,7 @@ sub receive_table {
 		}
 		push @message_cache, \@next;
 	}
+	return;
 }
 
 1;
@@ -111,33 +131,70 @@ threads::lite - Yet another threads library
 
 =head1 VERSION
 
-Version 0.021
+Version 0.022
 
 =head1 SYNOPSIS
 
-This module implements threads for perl. One crucial difference with normal threads is that the threads are B<entirely> disconnected, except by message queues (channel). It thus facilitates a message passing style of multi-threading.
+ use Modern::Perl;
+ use threads::lite qw/spawn self receive receive_table/;
 
-=head1 CLASS METHODS
+ sub child {
+ 	 my $other = threads::lite::receive;
+     while (<>) {
+		 chomp;
+         $other->send(line => $_);
+     }
+ }
+
+ my $child = spawn({ monitor => 1 } , \&child);
+ $child->send(self);
+
+ my $continue = 1;
+ while ($continue) {
+	 receive_table(
+		 [ 'line' ] => sub {
+			 my (undef, $line) = @_;
+			 say "received line: $line";
+		 },
+		 [ 'exit', qr//, $child->id ] => sub {
+			 say "received end of file";
+			 $continue = 0;
+		 },
+		 [] => sub {
+			 die sprintf "Got unknown message: (%s)", join ", ", @_;
+		 },
+	 );
+ };
+
+=head1 DESCRIPTION
+
+This module implements threads for perl. One crucial difference with normal threads is that the threads are disconnected, except by message queues (channel). It thus facilitates a message passing style of multi-threading.
+
+=head1 FUNCTIONS
+
+All these functions are exported optionally.
+
+=head2 Utility functions
 
 =head3 spawn($options, $sub)
 
 Spawn new threads. It will run $sub and send all monitoring processes it's return value. $options is a hashref that can contain the following elements.
 
-=over 4
+=over 2
 
-=item modules => [...]
+=item * modules => [...]
 
 Load the specified modules before running any code.
 
-=item pool_size => int
+=item * pool_size => int
 
 Create C<pool_size> identical clones. The threads are cloned right after module load time, but before any code is run.
 
-=item monitor => 0/1
+=item * monitor => 0/1
 
 If this is true, the calling process will monitor the newly spawned threads. Defaults to false.
 
-=item stack_size => int
+=item * stack_size => int
 
 The stack sze for the newly created threads. It defaults to 64 kiB.
 
@@ -145,13 +202,25 @@ The stack sze for the newly created threads. It defaults to 64 kiB.
 
 $sub can be a function name or a subref. If it is a name, you must make sure the module it is in is loaded in the new thread. If it is a reference it will be serialized and sent to the new thread. This means that any enclosed variables will probability not work as expected.
 
-=head1 FUNCTIONS
+=head3 self
 
-All these functions are exported by default.
+Retreive the tid corresponding with the current thread.
 
 =head2 Receiving functions
 
-threads::lite exports by default three functions for receiving messages from the process mailbox. Each of them accepts matching patterns used to select those messages. Each of the elements in the pattern will be smartmatched against the appropriate element in the message. The message may contain more elements than the pattern, they will not be included in the matching.
+threads::lite defines four functions for receiving messages from the thread's mailbox. Each of them accepts matching patterns used to select those messages. Pattern matching works like this:
+
+=over 2
+
+=item * If the pattern contains more elements than the message, the match fails.
+
+=item * If the pattern contains more elements than the message, the superfluous elements are ignored for the match.
+
+=item * Each of the elements in the message is smartmatched against the corresponding element in the pattern. Smartmatching semantics are defined in L<perlsyn|perlsyn/<"Smart-matching-in-detail">. If an element fails, the whole match fails.
+
+=item * When a match fails, the next message on the queue is tried.
+
+=back
 
 =head3 receive(@pattern)
 
@@ -161,15 +230,13 @@ Return the first message matching pattern @pattern. If there is no such message
 
 Return the first message matching pattern @pattern. If there is no such message in the queue, it returns an empty list (undef in scalar context).
 
-=head3 receive_table( [@pattern] => action ]...)
+=head3 receive_table( [@pattern] => action...)
 
-Try to match each pattern to the message queue. The first successful pattern is used. If none of the patterns match any of the messages in the queue, it blocks until a suitable message is received.
+This goes through each of the patterns until it can find one that matches a message on the queue, and call its action if it is defined. If none of the patterns match any of the messages in the queue, it blocks until a suitable message is received.
 
-=head2 Utility functions
+=head3 receive_table_nb( [@pattern] => action...)
 
-=head3 self
-
-Retreive the tid corresponding with the current thread.
+This goes through each of the patterns until it can find one that matches a message on the queue, and call its action if it is defined. If none of the patterns match any of the messages in the queue, it blocks until a suitable message is received. If none of the patterns match any of the messages in the queue, it return an empty list.
 
 =head1 AUTHOR
 
