@@ -4,9 +4,9 @@ use strict;
 use warnings;
 use Exporter 5.57 qw/import/;
 
-our @EXPORT_OK = qw/parallel_map/;
+our @EXPORT_OK = qw/parallel_map parallel_grep/;
 
-use threads::lite;
+use threads::lite qw/self spawn receive receive_table/;
 
 our $VERSION = $threads::lite::VERSION;
 
@@ -48,7 +48,7 @@ sub new {
 	);
 	my @modules = ('threads::lite::list', @{ $options{modules} });
 	my %threads = map { ( $_->id => $_ ) }
-		threads::lite->spawn({ modules => \@modules, monitor => 1 , pool_size => $options{threads}}, 'threads::lite::list::_mapper' );
+		spawn({ modules => \@modules, monitor => 1 , pool_size => $options{threads}}, 'threads::lite::list::_mapper' );
 	$_->send(filter => $options{code}) for values %threads;
 	return bless \%threads, $class;
 }
@@ -115,6 +115,8 @@ sub DESTROY {
 	my $self = shift;
 	for my $thread (values %{ $self }) {
 		$thread->send('kill');
+		receive('exit', qr//, $thread->id);
+		delete $self->{$thread->id};
 	}
 	return;
 }
@@ -127,7 +129,7 @@ threads::lite::list - Threaded list utilities
 
 =head1 VERSION
 
-Version 0.020
+Version 0.022
 
 =head1 SYNOPSIS
 
